# R-functions used for DESeq2 analysis single-cell RNA-seq data.
# In case of questions contact Jens Johansen - jens.johansen@bric.ku.dk

##############################################################
### DESEQ2 FUNCTIONS
##############################################################
##NB http://grokbase.com/t/r/bioconductor/13bpayfmwz/bioc-deseq2-foldchange-values-with-only-0s
#http://permalink.gmane.org/gmane.science.biology.informatics.conductor/51749
# log2FC!=0 even if compared levels only contain zero counts! 



merge_list_of_dataframes<-function(mylist,key) {
  #extract just DESeq2 'results' table from each
  
  #add contrast names to results before merging tables 
  for (i in 1:length(mylist)) {
    cur_nm<-names(mylist)[i]
    cur_cn<-colnames(mylist[[cur_nm]])
    j<-cur_cn!=key
    colnames(mylist[[cur_nm]])[j]<-paste(cur_nm,cur_cn[j],sep="_") 
  }
  #merge dataframes. keep missing all keys by padding with NAs
  totdat<-Reduce(function(x,y) { merge(x,y,by=key,all=TRUE) }, mylist)
  rownames(totdat)<-totdat$gene
  return(totdat)
}

##

deseq2.preprocess.plots<-function(y,lab="rnaseq") {
   #input must be class "DESeqDataSet" of 'DESeq' processed data

   if(class(y)!="DESeqDataSet") { stop("Input must be class 'DESeqDataSet' of 'DESeq' processed data") }

   library(tidyr)
   library(DESeq2)
   library(ggplot2)
   library(pheatmap)
   library(vsn)

   #plot sizeFactors - they should fairly similar - especially across groups being compared
   pdf(paste("sizefactors",lab,"pdf",sep="."),paper="a4")
   par(mar=c(5, 12, 4, 2) )
   barplot(sizeFactors(y),horiz=TRUE,las=2,main="SizeFactors")
   dev.off()
   
   #plot fraction of genes with zero count across samples - they should fairly similar - especially across groups being compared
   pdf(paste("fraction_zero_count_by_sample",lab,"pdf",sep="."),paper="a4")
   fraction_zero_count<-apply(counts(y),2,function(v) 100*(sum(v==0)/length(v)) )
   par(mar=c(5, 12, 4, 2) )
   barplot(fraction_zero_count,horiz=TRUE,las=2,xlab="Percent",main="Percent zero count genes")
   dev.off()
   
   #density plot of log2 raw counts+1
   p <- ggplot(aes(x=value,colour=key), data=gather(data.frame(log2(counts(y)+1)))) + geom_density() + ggtitle("log2+1 transformed raw counts")
   ggsave(filename=paste("density_log2_rawcounts",lab,"pdf",sep="."),p)
   #dev.off()
   
   #perform the two different transformations, that DESeq2 offer for plotting data
   #yr<-rlogTransformation(y,blind=TRUE)
   yv<-varianceStabilizingTransformation(y,blind=TRUE)

   #extract data frame with experiment factors
   coldata<-as.data.frame(colData(y))
   coldata<-coldata[,-match(c("sizeFactor","replaceable"),colnames(coldata),nomatch=0)]

   #make heatmap
   pdf(paste("heatmap_sample2sample_dist",lab,"pdf",sep="."),paper="a4")
   #mat<-as.matrix(dist(t(assay(yr))))
   mat<-as.matrix(cor(assay(yv)))
   #pheatmap(mat,annotation_col=coldata,cluster_rows=FALSE)
   pheatmap(mat)
   dev.off()

   #make one PCA plot per experiment factor (type)
   mytypes<-colnames(coldata)
   for (mytype in mytypes ) {
     p<-DESeq2::plotPCA(yv,intgroup=mytype,ntop=500)
     ggsave(filename=paste("PCA",mytype,lab,"pdf",sep="."),p)
     #dev.off()
   }

   #plot variance as function of mean after transformations
   pdf(paste("variance_stabilizing_transformations",lab,"pdf",sep="."),paper="a4",width=18)
   par(mfrow=c(1,3))
   notAllZero <- (rowSums(DESeq2::counts(y))>0)
   meanSdPlot(log2(DESeq2::counts(y,normalized=TRUE)[notAllZero,] + 1)) #,main="Shifted log. log2(n+1)")
   meanSdPlot(assay(yv[notAllZero,])) #,main="Regularized log (DESeq default)")
   meanSdPlot(assay(yv[notAllZero,])) # ,main="variance stabilizing")
   dev.off()

   pdf(file=paste("MAplot",lab,"pdf",sep="."),paper="a4")
   plotMA(y,ylim=c(-2,2),main="MA plot")
   dev.off()

   pdf(file=paste("dispersion",lab,"pdf",sep="."),paper="a4")
   plotDispEsts(y)
   dev.off()

   pdf(file=paste("sparsity",lab,"pdf",sep="."),paper="a4")
   plotSparsity(y) #x=rowSums,y=max(row)/rowSums
   dev.off()
   
   #plot cooks distance for the DESeq fitted data. NAs are removed
   p<-ggplot(gather(data.frame(assays(y)[["cooks"]])),aes(y=value,x=key)) + geom_boxplot() + ggtitle("Cook's distance")
   ggsave(paste("cooks_distance",lab,"pdf",sep="."),p)
   #dev.off()
}

##
fac2col<-function(vec) {
  vec<-as.factor(vec)
  cols<-rainbow(length(levels(vec)))
  mycols<-as.integer(vec)
  for(i in 1:length(cols)) {
    mycols[mycols==i]<-cols[i]
  }
  names(cols)<-levels(vec)
  return(list(col_vec=mycols,col_pal=cols))
}

##
make.pca.ggplot<-function(x,ann=NULL,pcs=c(1,2),color_fac=NULL,shape_fac=NULL,alpha_fac=NULL) {
   #'x' must be DESeq classes or simple numeric matrix 
   #'pcs': principal components to plot
   #'color_fac', 'shape_fac' and 'alpha_fac' must exist as column name in 'ann'

   library(ggplot2)
   library(DESeq2)

   #extract data matrix & annotation depending on class
   if(class(x)=="DESeqTransform") {
     mat<-assay(x)
     ann<-colData(x)
   } else if(class(x)=="DESeqDataSet") {
     mat<-counts(x,normalized=TRUE)
     ann<-colData(x)
   } else {
     mat<-x
   }

   #calculate the principal components
   pca <- prcomp(t(mat))
   #calculate how much variance of total data variance each PC captures
   percentVar <- round(100*(pca$sdev^2/sum(pca$sdev^2)),1)

   #make data frame with PCs and annotation used as input to ggplot
   dat<-data.frame(pca$x,ann)

   #plot requested principal components.
   #show how much variance each PC captures on the axis label
   pcs.lab<-paste("PC",pcs,sep="")
   g<-ggplot(dat, aes_string(pcs.lab[1],pcs.lab[2],color=color_fac,shape=shape_fac,alpha=alpha_fac)) + geom_point(size=3) + theme_bw()
   g<-g+xlab(paste(pcs.lab[1],":",percentVar[pcs[1]],"% variance"))+ ylab(paste(pcs.lab[2],":",percentVar[pcs[2]],"% variance"))

   return(g)
}

##
#plotCounts(y,gene="Rps6ka3",intgroup="Group") #
#prettier alternative to DESeq::plotCounts 
#y : DESeq object
plotCounts2<-function(y,gene,groupby,colorby) {
  d<-data.frame(colData(y),counts(y,normalized=TRUE)[gene,])
  colnames(d)[ncol(d)]<-gene
  ggplot(aes_string(x=groupby,y=gene,colour=colorby), data=d) + geom_point() + theme(axis.text.x=element_text(angle=90,hjust=1,vjust=0.5))
}

##

##plot direction of significant genes (up=1,down=-1). Non-significant genes are indicated by white 
#data frame from merging of multiple DESeq::results. Uses columns 'padj' and 'log2FoldChange'
plot_direction_signif_genes<-function(dat,fdr=0.1,log2fc=0,return=FALSE) {

  library(ggplot2)
  library(tidyr)
  
  #fixed colors even if a factor level is missing. https://stackoverflow.com/questions/43440068/ggplot2-fix-colors-to-factor-levels
  scale_fill_jens <- function(...){
      ggplot2:::manual_scale('fill', values = setNames(c('red', 'white', 'green'),c("-1","0","1")), ...)
  }

  #find unique genes with fdr below threshold in 1+ comparisons
  ind_signif<-sort(unique(unlist(apply(dat[,grep("padj$",colnames(dat))],2,function(v) which(v<=fdr) ))))
  z<-dat[ind_signif,]

  #make boolean FDR matrix of whether gene regulation is significant or not, based on fdr threshold
  z1<-z[,grep("padj$",colnames(z))]<=fdr
  z1[is.na(z1)]<-FALSE

  #make log2FC matrix with 3 levels: -1 (Down), 0 (NC), +1 (Up)
  w1<-z[,grep("log2FoldChange$",colnames(z))]
  w1[w1>=log2fc]<-1
  w1[w1<=(-log2fc)]<--1
  w1[!z1]<-0 #if not signif FDR set log2FC=0
  w1[w1<log2fc & w1>(-log2fc)]<-0 #if log2FC below threshold, set log2FC=0
  colnames(w1)<-sub(".log2FoldChange","",colnames(w1))
  #find genes, that do not pass log2fc filter for any comparison
  ind_zero<-apply(w1,1,function(v) all(v==0) )
  w1<-w1[!ind_zero,,drop=FALSE]
  
  if(nrow(w1)==0) {
    cat("No genes left\n")
  } else {
  
    #reshape to data for ggplot
    d1<-gather(data.frame(w1,Gene=rownames(w1)),key=Comparison,value=Direction,-Gene)
    d1$Direction<-factor(d1$Direction,levels=c("-1","0","1"))

    #order rows and columns by setting factor levels to clustering order
    ind_rows<-hclust(dist(w1))$order
    d1$Gene<-factor(d1$Gene,levels=rownames(w1)[ind_rows])
    ind_cols<-hclust(dist(t(w1)))$order
    d1$Comparison<-factor(d1$Comparison,levels=colnames(w1)[ind_cols])

    #make heatmap
    p<-ggplot(d1, aes(Comparison,Gene)) + geom_tile(aes(fill = Direction)) + scale_fill_jens() + theme(axis.text.x=element_text(angle=90,hjust=0.5,vjust=0.5))

    if(return) {
      return(list(dat=w1,p=p))
    }
  }
} 

##

#collect gene annnotation for mouse using 
#idtype is REFSEQ or ENSEMBL
get_mm_go_ann<-function(input_ann,idtype) {

  suppressPackageStartupMessages(library(tidyr))
  suppressPackageStartupMessages(library(org.Mm.eg.db))
  suppressPackageStartupMessages(library(GO.db))

  si<-sessionInfo()$otherPkgs
  cat("Using org.Mm.eg.db version",si$org.Mm.eg.db$Version,"\n")
  cat("Using GO.db version",si$GO.db$Version,"\n")
  
  #extra extra ann info from bioC package
  # columns(org.Mm.eg.db)
  # keytypes(org.Mm.eg.db)
  #keys(org.Mm.eg.db,keytype=idtype) #all refseq ids
  mykeys<-unique(as.character(input_ann[,"ID"]))
  
  cat("Gathering annotation for",length(mykeys),idtype,"IDs\n") 
  
  cat("  Getting gene description\n") 
  gene_ann_with_go<-select(org.Mm.eg.db, keys=mykeys,columns=c("SYMBOL","GENENAME","ALIAS","GO","ONTOLOGY"), keytype=idtype) 
  #'select()' returned 1:many mapping between keys and columns
  gene_ann_with_go<-gene_ann_with_go[,colnames(gene_ann_with_go)!="EVIDENCE"]
  gene_ann_with_go<-gene_ann_with_go[!duplicated(gene_ann_with_go),] 

  #get go ID description from GO.db
  cat("  Getting gene GOs\n") 
  go_ann<-select(GO.db,keys=unique(as.character(gene_ann_with_go[,"GO"])),keytype="GOID",columns="TERM") 
  go_ann<-data.frame(GOID=go_ann[,"GOID"],GOterm=apply(go_ann,1,paste,collapse=":"))
  #'select()' returned 1:1 mapping between keys and columns

  #merge hgu95av2.db and GO.db annotation
  cat("  Merging tables\n") 
  gene_ann_with_go2<-merge(x=gene_ann_with_go,y=go_ann,by.x="GO",by.y="GOID") 

  #merge with original input
  ann_full<-merge(x=input_ann,y=gene_ann_with_go2,by.x="ID",by.y=idtype)
  #columns: GO + SYMBOL not needed anymore
  ann_full<-ann_full[,-match(c("GO","SYMBOL"),colnames(ann_full))] 

  #apply(ann_full,2,function(v) any(is.na(v)) )
  #      ID   Symbol GENENAME    ALIAS ONTOLOGY   GOterm 
  #    FALSE    FALSE     TRUE     TRUE     TRUE    FALSE 

  #rm NAs including potential NA-only rows
  ann_full<-apply(ann_full,2,function(v) { v[is.na(v)]<-""; v[v=="NA:NA"]<-""; return(v) } )
  #apply(zz,2,function(v) sum(is.na(v)) )

  #collapse rows on gene symbol
  cat("  Collapsing table by Symbol name\n") 
  ann_collapsed<-aggregate(ann_full,by=list(Symbol=factor(ann_full[,"Symbol"])), function(v) paste(sort(unique(v)),collapse=",") )
  ann_collapsed<-apply(ann_collapsed,2,function(v) { v=sub("^,","",v); return(sub(",$","",v)) } ) #remove leading comma in collapsed rows
 
  cat("  Returning table for ",nrow(ann_collapsed),"unique symbols\n")
 
  return(ann_collapsed)

} 

####

myDESeqHexbinMAplot<-function(deseqresults,padj_threshold=0.1,nbins=100,bincolors=c("blue","white")) {
#produces a MA-plot. Close points are binned using "hexbin". Points are colored according to number of point in a bin. This function is very useful when plotting many points.
#It is possible to give a reference array (e.g. median array, which is the default setting) to compare to
#possible to compare just within groups
#assumes logged input data

#either list if deseq results or single deseq result
if( is.list(deseqresults) && all(unlist(lapply(deseqresults,class))=="DESeqResults") ) {
  #combine results head2tail in single dataframe
  dat<-c()
  for(i in 1:length(deseqresults)) {
    cur_dat<-data.frame(Gene=rownames(deseqresults[[i]]),Comparison=names(deseqresults)[i],as.data.frame(deseqresults[[i]][,c("baseMean","log2FoldChange","padj")]))
    dat<-rbind(dat,cur_dat)
  }  
  p<-hexbinMAplot(dat=dat,xname="baseMean",yname="log2FoldChange",scalex="log2",nbins=100,bincolors=c("blue","white"),threshold=padj_threshold,thresholdname="padj",facetwrapname="Comparison")
} else if(class(deseqresults)=="DESeqResults") {
  dat<-as.data.frame(deseqresults)
  #filter out data with zero for all samples
  dat<-subset(dat,baseMean!=0)
  p<-hexbinMAplot(dat=dat,xname="baseMean",yname="log2FoldChange",scalex="log2",nbins=nbins,bincolors=bincolors,threshold=padj_threshold,thresholdname="padj")
} else {
  cat("Input must be of class DESeqResults\n")
}

return(p)    

} 

###

hexbinMAplot<-function(dat,xname,yname,nbins=100,bincolors=c("blue","white"),scalex=NULL,scaley=NULL,threshold=NULL,thresholdname=NULL,facetwrapname=NULL) {

  library(ggplot2)
  library(hexbin)
  
  #make constant and symmetric ylim values
  ylimrange<-max(abs(dat[,yname]))*c(-1.01,1.01)

  #build ggplot
  p<-ggplot(dat, aes_string(x=xname, y=yname)) + 
      #scale_x_log() +
      geom_hex(bins=nbins,show.legend=FALSE) + 
      geom_hline(yintercept=0,color="black") + 
      ylim(ylimrange[1],ylimrange[2]) + 
      theme_bw() +
      #set color scale
      scale_fill_gradientn(colours=bincolors,name = "Frequency",na.value=NA) +
      #add line of loess fit 
      geom_smooth(color="orange") 
 
  if(!is.null(scalex)) { 
    p <- p + scale_x_continuous(trans = scalex)
  }

  if(!is.null(scaley)) { 
    p <- p + scale_y_continuous(trans = scaley)
  }
  
  if(!is.null(threshold)) {
      #add red points of significant genes
      p<-p + geom_point(data=dat[which(dat[,thresholdname]<threshold),],colour="red",size=2) 
  }

  if(!is.null(facetwrapname)) {
    p <- p + facet_wrap(facetwrapname) 
  }

  return(p)
} 



###

#'myresults' - expected to be dataframe or DESeqResults object
#must contain columns 'padj','pvalue','log2FoldChange'
#could make custom transform function fot -log10(): http://freerangestats.info/blog/2015/09/05/creating-a-scale-transformation
volcano_ggplot2<-function(myresults,topN=10,nbins=100,bincolors=c("blue","white")) {

  library(ggplot2)
  library(ggrepel)
  
  if(is.list(myresults)) {
    results<-do.call(rbind,lapply(myresults,function(ll) data.frame(gene=rownames(ll),ll) ))
    results<-data.frame(comparison=sub("\\.[^\\.]+$","",rownames(results)),results)
  } else {
    results<-data.frame(myresults,gene=rownames(myresults))
  }
  
  results = data.frame(results,Signif=ifelse(results$padj<0.05, "FDR<0.05", "Not Sig"),log10=-1*log10(results$pvalue))
  results <- results[order(results$padj),]
  
  #build ggplot
  p<-ggplot(subset(results,padj>0.05), aes(x=log2FoldChange, y=log10)) + 
      geom_hex(bins=nbins,show.legend=FALSE) + 
      geom_vline(xintercept=0,color="grey",lty=2) + 
      xlim(c(-1,1)*max(abs(results$log2FoldChange))) +
      theme_bw() +
      #set color scale
      scale_fill_gradientn(colours=bincolors,name = "Frequency",na.value=NA) +
      ylab(label="-log10(pvalue)") + xlab(label="log2(fold change)")

  #plot topN or if less than topN, all the significant genes      
  topN <- ifelse(topN>=sum(results$padj<=0.05,na.rm=TRUE),sum(results$padj<0.05,na.rm=TRUE),topN)     

  signif_results<-subset(results,padj<=0.05,drop=FALSE)
  if(nrow(signif_results)>0) {
    p <- p + geom_point(data=signif_results,colour="red",size=2) 
    p <- p + ggrepel::geom_text_repel(data=signif_results[1:topN, ], ggplot2::aes(label=gene))
      if(any(colnames(results)=="comparison")) {
        p <- p + facet_wrap(~comparison, scales = "free_y")
      }
  } else {
    cat("No significant genes\n")
  }
    
  return(p)
  
} 

###

#myfit: deseq2 fitted object
#compare_list: list of lists with length 1 or 2 with names 'mycontrasts' (for 'contrast' arg in results()) and/or 'mynames' (for 'name' arg in results())

extract_deseq2_results<-function(myfit,compare_lists,lab="deseq2_results",altHypothesis="greaterAbs",lfcThreshold=0,alpha=0.1){
 
  library(DESeq2)
  library(ggplot2)
  library(tidyr)
 
  int_run_multiple_deseq<-function(myfit,compare_lists,altHypothesis="greaterAbs",lfcThreshold=0,alpha=0.1) {
    
    reslist<-list()
    if(any(names(compare_lists) %in% "mycontrasts")) {
      reslist<-c(reslist,lapply(compare_lists$mycontrasts,function(cur_contrast) results(myfit,contrast=cur_contrast,altHypothesis=altHypothesis,lfcThreshold=lfcThreshold,alpha=alpha) ))
    }
    if(any(names(compare_lists) %in% "mynames")) {
      reslist<-c(reslist,lapply(compare_lists$mynames,function(cur_name) results(myfit,name=cur_name,altHypothesis=altHypothesis,lfcThreshold=lfcThreshold,alpha=alpha) ))
    } 

    #combine to results in single dataframe
    full_dat<-data.frame(mcols(myfit)[,1:9])
    #for each comparison
    for(i in 1:length(reslist)) {
      cat("Contrast number:",i,"\n")
      cur_dat<-as.data.frame(reslist[[i]])[,c("log2FoldChange","lfcSE","pvalue","padj")]
      colnames(cur_dat)<-paste(names(reslist)[i],colnames(cur_dat),sep=".")
      full_dat<-cbind(full_dat,cur_dat)
    }

    return(list(reslist=reslist,full_dat=full_dat))
  }
  
  #extract DESeq results for each comparison into list
  cat("  Extracting results\n")
  res <- int_run_multiple_deseq(myfit=myfit,compare_lists=compare_lists,altHypothesis=altHypothesis,lfcThreshold=lfcThreshold,alpha=alpha) 
  
  #make MA-plots for each comparison
#   cat("  MA plots\n")
#   p<-myDESeqHexbinMAplot(res$reslist,padj_threshold=0.1,nbins=100,bincolors=c("blue","white")) + ggtitle(label="MA plot",subtitle=lab)
#   ggsave(paste("MAplots_hexbin_",lab,".pdf",sep=""),p)
  #dev.off()
  
  #make volcano plots
#   cat("  Volcano plots\n")
#   p<-volcano_ggplot2(res$reslist) + ggtitle(label="Volcano plot",subtitle=lab)
#   ggsave(paste("volcano_plot_",lab,".pdf",sep=""),p) 
  #dev.off()
  
  #make DE direction plots for all comparisons to easily see degree of overlap between comparisons
  cat("  DE direction plots\n")
  p<-plot_direction_signif_genes(dat=res$full_dat,fdr=0.01,log2fc=1,return=TRUE)$p + ggtitle(label="DE direction plot",subtitle=lab)
  ggsave(paste("direction_signif_genes_",lab,".pdf",sep=""),p) 
  #dev.off()
 
  #make histograms of p-value distributions for all results
  #pvalue hist only meaningful for lfcThreshold=0, so recalculate just for these plots if lfcThreshold!=0
  cat("  p-value histograms\n")
  if (lfcThreshold==0) {
    p<-ggplot(gather(res$full_dat[,grep("pvalue",colnames(res$full_dat))],key="Comparison",value="pvalue"), aes(x=pvalue)) + geom_histogram() + facet_wrap( ~Comparison) + theme_bw() + ggtitle(label=paste("altHypothesis:",altHypothesis,"; lfcThreshold:",lfcThreshold,"; alpha:",alpha),subtitle=lab)
    ggsave(paste("pvalue_hist_",lab,".pdf",sep=""),p)
    #dev.off()
  } else {
    cat("Recalculating with lfcThreshold=0 for p-value histograms\n")
    pvalres <- int_run_multiple_deseq(myfit=myfit,compare_lists=compare_lists,altHypothesis="greaterAbs",lfcThreshold=0,alpha=0.1)
    p<-ggplot(gather(pvalres$full_dat[,grep("pvalue",colnames(pvalres$full_dat))],key="Comparison",value="pvalue"), aes(x=pvalue)) + geom_histogram() + facet_wrap( ~Comparison) + theme_bw() + ggtitle(label=paste("altHypothesis:",altHypothesis,"; lfcThreshold:",lfcThreshold,"; alpha:",alpha),subtitle=lab)
    ggsave(paste("pvalue_hist_",lab,".pdf",sep=""),p)
    #dev.off()
  }
  
  #plot results of independent filtering
  cat("  Independent filtering plots\n")
  filter_list1<-lapply(res$reslist,function(res) data.frame(metadata(res)$filterNumRej,do.call(cbind,metadata(res)$lo.fit)
  ,filterTheta=metadata(res)$filterTheta) )
  filter_dat1<-do.call(rbind,filter_list1)
  filter_dat1<-data.frame(filter_dat1,comparison=sub("\\.\\d+\\.{0,1}\\d{0,}\\%$","",rownames(filter_dat1)))
  p<-ggplot(filter_dat1,aes(x=theta,y=numRej)) + geom_point(color="grey") + facet_wrap(~comparison) + theme_bw() + geom_vline(aes(xintercept = filterTheta),color="red",lty=2) + geom_line(aes(x=x,y=y),color="red") + ggtitle("Independent filtering",subtitle=lab)
  ggsave(paste("independent_filtering_",lab,".pdf",sep=""),p)
  #dev.off()

  #determine number of DE genes as function of padj threshold
  cat("  Number of DEs as function of padj threshold\n")
  padj.threshold<-seq(0.001,0.1,by=0.001)
  nsignif_dat<-data.frame(padj.threshold=padj.threshold,do.call(cbind,lapply(res$reslist,function(m) sapply(padj.threshold,function(t) sum(m$padj<=t,na.rm=TRUE) ) )))
  nsignif_dat_long<-gather(nsignif_dat,comparison,N,-padj.threshold)
  #colnames(nsignif_dat_long)[2]<-"comparison"
  p<-ggplot(nsignif_dat_long,aes(x=padj.threshold,y=N)) + geom_line() + ylab("Number of DE genes") + theme_bw() + facet_wrap(~comparison)
  ggsave(paste("nsignif_vs_padjthreshold_",lab,".pdf",sep=""),p)
  #dev.off()
  
  #
  full_dat<-res$full_dat
  
  #calculate groupmeans
  if(any(colnames(colData(myfit)) %in% "Group")) {
    cat("  Adding Group means to dataframe\n")
    groupmean<-t(apply(counts(myfit,normalized=TRUE),1, function(v) tapply(v,list(group=colData(myfit)$Group),mean )))
    full_dat<-data.frame(full_dat,groupmean)
  }
  
  #rm GO-ann that seems messed up
  #full_dat<-full_dat[,-match(c("ONTOLOGY","GOterm"),colnames(full_dat))]
  
  #write to file
  cat("  Writing to file\n")
  write.table(x=full_dat,file=paste("deseq2_",lab,".txt",sep=""),quote=FALSE, sep="\t", row.names=FALSE)
 
  return(full_dat)
 
} 

## prepare data for ggplot'ting
deseq2_res_dat2ggplot_melt<-function(type_nm,dat) {

  dat<-data.frame(Gene=dat$Symbol,dat[,grep(type_nm,colnames(dat))],stringsAsFactors=F)
  colnames(dat)<-sub(paste("_",type_nm,sep=""),"",colnames(dat))
  colnames(dat)<-sub("^X","",colnames(dat))

  ##make ggplot ready (melted) dataframe
  melt_dat<-melt(dat) #
  colnames(melt_dat)<-c("Gene","Stage",type_nm)
  #convert Gene to character to allow normal sorting
  melt_dat$Gene<-as.character(melt_dat$Gene)
  melt_dat$Stage<-factor(as.character(melt_dat$Stage),levels=c("MII","2c","4c","8c"))

  #add additional columns of factors converted to integers. Use integer levels to sort/rearrange order in ggplot.
  #Need to reorder, if at later point use droplevels() or similar
  melt_dat <- data.frame(melt_dat,Stage2int=as.integer(melt_dat$Stage),key=apply(melt_dat[,c("Gene","Stage")],1,paste,collapse="_"),stringsAsFactors=FALSE) %>% arrange(Gene)
  
  return(melt_dat)

}





